/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_RTCConfigurationBinding_h__
#define mozilla_dom_RTCConfigurationBinding_h__

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

class NativePropertyHooks;
struct RTCConfigurationAtoms;
struct RTCIceServer;
struct RTCIceServerAtoms;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct RTCIceServer : public MainThreadDictionaryBase
{
  nsString mCredential;
  Optional<nsString > mUrl;
  nsString mUsername;

  inline RTCIceServer()
  {
  }

private:
  RTCIceServer(const RTCIceServer&) MOZ_DELETE;
  void operator=(const RTCIceServer) MOZ_DELETE;

  static bool
  InitIds(JSContext* cx, RTCIceServerAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObject(JSContext* cx, JS::Handle<JSObject*> parentObject, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);
};
struct RTCIceServerInitializer : public RTCIceServer
{
  RTCIceServerInitializer();
};


struct RTCConfiguration : public MainThreadDictionaryBase
{
  Optional<Sequence<RTCIceServer > > mIceServers;

  inline RTCConfiguration()
  {
  }

private:
  RTCConfiguration(const RTCConfiguration&) MOZ_DELETE;
  void operator=(const RTCConfiguration) MOZ_DELETE;

  static bool
  InitIds(JSContext* cx, RTCConfigurationAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObject(JSContext* cx, JS::Handle<JSObject*> parentObject, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);
};
struct RTCConfigurationInitializer : public RTCConfiguration
{
  RTCConfigurationInitializer();
};


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_RTCConfigurationBinding_h__
