/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_InspectorUtilsBinding_h__
#define mozilla_dom_InspectorUtilsBinding_h__

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"

namespace mozilla {
namespace dom {

struct InspectorRGBTripleAtoms;
class NativePropertyHooks;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct InspectorRGBTriple : public MainThreadDictionaryBase
{
  uint8_t mB;
  uint8_t mG;
  uint8_t mR;

  inline InspectorRGBTriple()
  {
  }

private:
  InspectorRGBTriple(const InspectorRGBTriple&) MOZ_DELETE;
  void operator=(const InspectorRGBTriple) MOZ_DELETE;

  static bool
  InitIds(JSContext* cx, InspectorRGBTripleAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObject(JSContext* cx, JS::Handle<JSObject*> parentObject, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);
};
struct InspectorRGBTripleInitializer : public InspectorRGBTriple
{
  InspectorRGBTripleInitializer();
};


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_InspectorUtilsBinding_h__
