/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_MediaStreamEventBinding_h__
#define mozilla_dom_MediaStreamEventBinding_h__

#include "DOMMediaStream.h"
#include "EventBinding.h"
#include "js/RootingAPI.h"
#include "jsapi.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/BindingUtils.h"
#include "mozilla/dom/CallbackInterface.h"
#include "mozilla/dom/Nullable.h"
#include "nsDOMEvent.h"

class JSObject;

namespace mozilla {

class DOMMediaStream;

namespace dom {

class MediaStreamEvent;
struct MediaStreamEventInit;
struct MediaStreamEventInitAtoms;
class NativePropertyHooks;

} // namespace dom

} // namespace mozilla

namespace mozilla {
namespace dom {

struct MediaStreamEventInit : public EventInit
{
  nsRefPtr<mozilla::DOMMediaStream> mStream;

  inline MediaStreamEventInit()
  {
  }

private:
  MediaStreamEventInit(const MediaStreamEventInit&) MOZ_DELETE;
  void operator=(const MediaStreamEventInit) MOZ_DELETE;

  static bool
  InitIds(JSContext* cx, MediaStreamEventInitAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObject(JSContext* cx, JS::Handle<JSObject*> parentObject, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);
};
struct MediaStreamEventInitInitializer : public MediaStreamEventInit
{
  MediaStreamEventInitInitializer();
};


namespace MediaStreamEventBinding {

  typedef mozilla::dom::MediaStreamEvent NativeType;

  // We declare this as an array so that retrieving a pointer to this
  // binding's property hooks only requires compile/link-time resolvable
  // address arithmetic.  Declaring it as a pointer instead would require
  // doing a run-time load to fetch a pointer to this binding's property
  // hooks.  And then structures which embedded a pointer to this structure
  // would require a run-time load for proper initialization, which would
  // then induce static constructors.  Lots of static constructors.
  extern const NativePropertyHooks sNativePropertyHooks[];
  JSObject*
  DefineDOMInterface(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Handle<jsid> id, bool aDefineOnGlobal);

  bool
  ConstructorEnabled(JSContext* /* unused */, JS::Handle<JSObject*> /* unused */);

  JSClass*
  GetJSClass();

  JSObject*
  Wrap(JSContext* aCx, JS::Handle<JSObject*> aScope, mozilla::dom::MediaStreamEvent* aObject, nsWrapperCache* aCache);

  template <class T>
  inline JSObject* Wrap(JSContext* aCx, JS::Handle<JSObject*> aScope, T* aObject)
  {
    return Wrap(aCx, aScope, aObject, aObject);
  }

  void
  CreateInterfaceObjects(JSContext* aCx, JS::Handle<JSObject*> aGlobal, JS::Heap<JSObject*>* aProtoAndIfaceArray, bool aDefineOnGlobal);

  JS::Handle<JSObject*>
  GetProtoObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal);

  JS::Handle<JSObject*>
  GetConstructorObject(JSContext* aCx, JS::Handle<JSObject*> aGlobal, bool aDefineOnGlobal = true);

} // namespace MediaStreamEventBinding



class MediaStreamEventJSImpl : public CallbackInterface
{
public:
  explicit inline MediaStreamEventJSImpl(JSObject* aCallback)
    : CallbackInterface(aCallback)
  {
  }

  void __Init(const nsAString& type, const MediaStreamEventInit& eventInitDict, ErrorResult& aRv, ExceptionHandling aExceptionHandling = eReportExceptions);

  already_AddRefed<DOMMediaStream> GetStream(ErrorResult& aRv, ExceptionHandling aExceptionHandling = eReportExceptions);
};


class MediaStreamEvent MOZ_FINAL : public nsDOMEvent
{
public:
  NS_DECL_ISUPPORTS_INHERITED
  NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(MediaStreamEvent, nsDOMEvent)

private:
  nsRefPtr<MediaStreamEventJSImpl> mImpl;
  nsCOMPtr<nsISupports> mParent;

public:
  MediaStreamEvent(JS::Handle<JSObject*> aJSImplObject, nsPIDOMWindow* aParent);

  nsISupports* GetParentObject() const;

  virtual JSObject* WrapObject(JSContext* aCx, JS::Handle<JSObject*> aScope) MOZ_OVERRIDE;

  static already_AddRefed<MediaStreamEvent> Constructor(const GlobalObject& global, JSContext* cx, const nsAString& type, const MediaStreamEventInit& eventInitDict, ErrorResult& aRv);

  // Mark this as resultNotAddRefed to return raw pointers
  already_AddRefed<DOMMediaStream> GetStream(ErrorResult& aRv) const;

  static bool
  _Create(JSContext* cx, unsigned argc, JS::Value* vp);
};


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_MediaStreamEventBinding_h__
